﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Web;
using System.Xml.Linq;

namespace Dynamicweb.Ecommerce.LiveIntegration.Configuration
{
    internal class ConfigurationHandler
    {        
        private static DateTime _configFileLastModifiedTime = DateTime.MinValue;
        private static Dictionary<string, Dictionary<string, string>> _config = null;

        private static object lockObject = new object();

        private static string CONFIG_FILE_PATH
        {
            get { return string.Format("/Files/System/LiveIntegration/{0}.Setup.xml", Constants.AddInName); }
        }

        public static Dictionary<string, string> GetConfig(string ConfigurationPath)
        {
            lock (lockObject)
            {
                FileInfo fi = new FileInfo(HttpContext.Current.Server.MapPath(CONFIG_FILE_PATH));
                if (!fi.Exists)
                    throw new FileNotFoundException("Configuration file does not exist. Please rerun setup.");

                if (_config == null)
                    _config = new Dictionary<string, Dictionary<string, string>>();

                if (fi.LastWriteTime.CompareTo(_configFileLastModifiedTime) > 0)
                    ReloadConfigFile(fi);

                if (!_config.ContainsKey(ConfigurationPath))
                    return null;

                return _config[ConfigurationPath];
            }
        }

        public static void SetConfig(string ConfigurationPath, Dictionary<string, string> values)
        {
            lock (lockObject)
            {
                FileInfo fi = new FileInfo(HttpContext.Current.Server.MapPath(CONFIG_FILE_PATH));
                if (fi.Exists)
                    ReloadConfigFile(fi);

                var xml = new XDocument();
                xml.Declaration = new XDeclaration("1.0", "utf-8", "yes");
                var root = new XElement(XName.Get("Configurations"));
                xml.Add(root);

                if (_config == null)
                    _config = new Dictionary<string, Dictionary<string, string>>();

                if (_config.ContainsKey(ConfigurationPath))
                    _config[ConfigurationPath] = values;
                else
                    _config.Add(ConfigurationPath, values);

                foreach (var configPath in _config.Keys)
                {
                    var configNode = new XElement(XName.Get(configPath));
                    root.Add(configNode);
                    foreach (var config in _config[configPath].Keys)
                        configNode.SetElementValue(XName.Get(config), _config[configPath][config]);
                }

                if (!fi.Directory.Exists)
                    fi.Directory.Create();

                if (fi.Exists)
                    fi.Delete();

                using (var writer = fi.CreateText())
                    writer.Write(xml.ToString());

            }
        }

        private static void ReloadConfigFile(FileInfo fi)
        {
            _config = new Dictionary<string, Dictionary<string, string>>();
            using (var reader = fi.OpenText())
            {
                var xDoc = XElement.Parse(reader.ReadToEnd());
                foreach (XElement configuration in xDoc.Nodes())
                {
                    var name = configuration.Name.LocalName;

                    if (!_config.ContainsKey(name))
                        _config.Add(name, new Dictionary<string, string>());

                    foreach (XElement configValue in configuration.Nodes())
                    {
                        var configValueName = configValue.Name.LocalName;
                        var value = configValue.Value;
                        _config[name].Add(configValueName, value);
                    }
                }
            }
            _configFileLastModifiedTime = fi.LastWriteTime;
        }
    }
}
